##
#
# Generic setup header for all simulations.
#
# Copyright 2007-2008, Freie Universitaet Berlin. All rights reserved.
# Please refer to the COPYING file for detailed licensing information.
#
# Author: Georg Wittenburg
#
# $Id: header.tcl 2583 2008-03-05 12:30:40Z wittenbu $
#
##



# Create God
set god_ [create-god $val(nn)]
$god_ on

# Create simulator
set ns_ [new Simulator]

# Initialize random number generator with new seed (0 -> random seed)
set rng_ [new RNG]
$rng_ seed 0

# Create channel
set chan_ [new $val(chan)]

# Create topography
set topo_ [new Topography]
$topo_ load_flatgrid $val(x) $val(y)

# Setup node configuration
$ns_ node-config \
	-adhocRouting $val(rp) \
	-llType $val(ll) \
	-macType $val(mac) \
	-ifqType $val(ifq) \
	-ifqLen $val(ifqlen) \
	-antType $val(ant) \
	-propType $val(prop) \
	-phyType $val(netif) \
	-topoInstance $topo_ \
	-agentTrace OFF \
	-routerTrace OFF \
	-macTrace OFF \
	-movementTrace OFF \
	-channel $chan_

# Enable heuristic random seed in shadowing radio propagation.
if {[$ns_ set propType_] == "Propagation/Shadowing"} then {
	[$ns_ set propInstance_] seed heuristic 0
}

# Open trace files
#set tracefd_	[open out.tr w]
set tracefd_	[open /dev/null w]
$ns_ trace-all $tracefd_
#set namtracefd_ [open out.nam w]
set namtracefd_ [open /dev/null w]
$ns_ namtrace-all-wireless $namtracefd_ $val(x) $val(y)

# Remember start time
set start_time_ [clock seconds]

# Begin and end of start, simulation, stop and finalize phases.
set start_phase_begin		0.0
set start_phase_end		[expr $start_phase_begin + 600.0]
set simulation_begin		$start_phase_end
set simulation_end		[expr $simulation_begin + $simulation_duration]
set stop_phase_begin		$simulation_end
set stop_phase_end		[expr $stop_phase_begin + 600.0]
set finalize_phase_begin	[expr $stop_phase_end + 3600.0]
set finalize_phase_end		[expr $finalize_phase_begin + 600.0]



$ns_ at [expr $finalize_phase_end] "stop"

proc stop {} {
	global ns_ tracefd_ namtracefd_ start_time_
	$ns_ halt
	$ns_ nam-end-wireless [$ns_ now]
	$ns_ flush-trace
	close $tracefd_
	close $namtracefd_
	set elapased_time [expr [clock seconds] - $start_time_ - 3600]	;# Formatting miraculously adds one hour.
	puts [format "Simulation complete (%s)." [clock format $elapased_time -format "%T"]]
	puts ""
	#printProcessStatus
	#puts ""
	printStats
}

proc printProcessStatus {} {
	set stats [open /proc/self/status r]
	while { [gets $stats line] >= 0 } {
		puts $line
	}
	close $stats
}

proc printStats {} {
	global val application_ node_

	set MAC_transmittedPackets				0
	set MAC_receivedPackets					0
	set MAC_droppedPackets_capture				0
	set MAC_droppedPackets_collision			0
	set MAC_droppedPackets_transmit				0
	set MAC_droppedPackets_bit_error			0
	set MAC_transmittedBytes				wide(0)
	set MAC_receivedBytes					wide(0)

	set APP_transmittedUnicastPackets			0
	set APP_receivedUnicastPackets				0
	set APP_unicastPDR					0.0
	set APP_transmittedBroadcastPackets			0
	set APP_receivedBroadcastPackets			0
	set APP_broadcastPDR					0.0
	set APP_transmittedBytes				wide(0)
	set APP_receivedBytes					wide(0)

	set SP_DEBUG_ERRORs					0
	set SP_DEBUG_WARNs					0

	set min_node_degree					255
	set max_node_degree					0
	set SP_direct_neighbors					0
	set avg_node_degree					0.0
	set SP_direct_neighbors_list				[list]
	set med_node_degree					0.0

	set SP_longest_route					0
	set SP_average_route					0.0

	set SP_RUDP_total_transmissions				0
	set SP_RUDP_successful_transmissions			0
	set SP_RUDP_pdr						0.0

	set SP_ICMP_transmitted_nops				0
	set SP_ICMP_received_nops				0

	set SP_SLP_total_queries				0
	set SP_SLP_successful_queries				0
	set SP_SLP_recall					0.0

	set SP_SERVICE_total_data_requests			0
	set SP_SERVICE_successful_data_requests			0
	set SP_SERVICE_data_request_recall			0.0
	set SP_SERVICE_total_successful_data_request_delay	0.0
	set SP_SERVICE_average_successful_data_request_delay	0.0
	set SP_SERVICE_total_data_updates			0
	set SP_SERVICE_successful_data_updates			0
	set SP_SERVICE_data_update_recall			0.0
	set SP_SERVICE_total_stream_requests			0
	set SP_SERVICE_successful_stream_requests		0
	set SP_SERVICE_stream_request_recall			0.0
	set SP_SERVICE_expected_stream_packets			0
	set SP_SERVICE_received_stream_packets			0
	set SP_SERVICE_stream_packet_recall			0.0
	set SP_SERVICE_total_replications			0
	set SP_SERVICE_successful_replications			0
	set SP_SERVICE_replication_success_rate			0.0
	set SP_SERVICE_total_migrations				0
	set SP_SERVICE_successful_migrations			0
	set SP_SERVICE_migration_success_rate			0.0

	for {set i 0} {$i < $val(nn)} {incr i} {
		set MAC_transmittedPackets [expr $MAC_transmittedPackets + [[$node_($i) set mac_(0)] set transmittedPackets_]]
		set MAC_receivedPackets [expr $MAC_receivedPackets + [[$node_($i) set mac_(0)] set receivedPackets_]]
		set MAC_droppedPackets_capture [expr $MAC_droppedPackets_capture + [[$node_($i) set mac_(0)] set droppedPackets_capture_]]
		set MAC_droppedPackets_collision [expr $MAC_droppedPackets_collision + [[$node_($i) set mac_(0)] set droppedPackets_collision_]]
		set MAC_droppedPackets_transmit [expr $MAC_droppedPackets_transmit + [[$node_($i) set mac_(0)] set droppedPackets_transmit_]]
		set MAC_droppedPackets_bit_error [expr $MAC_droppedPackets_bit_error + [[$node_($i) set mac_(0)] set droppedPackets_bit_error_]]
		set MAC_transmittedBytes [expr $MAC_transmittedBytes + [[$node_($i) set mac_(0)] set transmittedBytes_]]
		set MAC_receivedBytes [expr $MAC_receivedBytes + [[$node_($i) set mac_(0)] set receivedBytes_]]

		set APP_transmittedUnicastPackets [expr $APP_transmittedUnicastPackets + [$application_($i) set transmittedUnicastPackets_]]
		set APP_receivedUnicastPackets [expr $APP_receivedUnicastPackets + [$application_($i) set receivedUnicastPackets_]]
		set APP_transmittedBroadcastPackets [expr $APP_transmittedBroadcastPackets + [$application_($i) set transmittedBroadcastPackets_]]
		set APP_receivedBroadcastPackets [expr $APP_receivedBroadcastPackets + [$application_($i) set receivedBroadcastPackets_]]
		set APP_transmittedBytes [expr $APP_transmittedBytes + [$application_($i) set transmittedBytes_]]
		set APP_receivedBytes [expr $APP_receivedBytes + [$application_($i) set receivedBytes_]]

		set SP_DEBUG_ERRORs [expr $SP_DEBUG_ERRORs + [$application_($i) set SP_DEBUG_ERRORs]]
		set SP_DEBUG_WARNs [expr $SP_DEBUG_WARNs + [$application_($i) set SP_DEBUG_WARNs]]

		if {$min_node_degree > [$application_($i) set SP_direct_neighbors]} then {
			set min_node_degree [$application_($i) set SP_direct_neighbors]
		}
		if {$max_node_degree < [$application_($i) set SP_direct_neighbors]} then {
			set max_node_degree [$application_($i) set SP_direct_neighbors]
		}
		set SP_direct_neighbors [expr $SP_direct_neighbors + [$application_($i) set SP_direct_neighbors]]
		lappend SP_direct_neighbors_list [$application_($i) set SP_direct_neighbors]

		if {$SP_longest_route < [$application_($i) set SP_longest_route]} then {
			set SP_longest_route [$application_($i) set SP_longest_route]
		}
		set SP_average_route [expr $SP_average_route + [$application_($i) set SP_average_route]]

		set SP_RUDP_total_transmissions [expr $SP_RUDP_total_transmissions + [$application_($i) set SP_RUDP_total_transmissions]]
		set SP_RUDP_successful_transmissions [expr $SP_RUDP_successful_transmissions + [$application_($i) set SP_RUDP_successful_transmissions]]

		set SP_ICMP_transmitted_nops [expr $SP_ICMP_transmitted_nops + [$application_($i) set SP_ICMP_transmitted_nops]]
		set SP_ICMP_received_nops [expr $SP_ICMP_received_nops + [$application_($i) set SP_ICMP_received_nops]]

		set SP_SLP_total_queries [expr $SP_SLP_total_queries + [$application_($i) set SP_SLP_total_queries]]
		set SP_SLP_successful_queries [expr $SP_SLP_successful_queries + [$application_($i) set SP_SLP_successful_queries]]

		set SP_SERVICE_total_data_requests [expr $SP_SERVICE_total_data_requests + [$application_($i) set SP_SERVICE_total_data_requests]]
		set SP_SERVICE_successful_data_requests [expr $SP_SERVICE_successful_data_requests + [$application_($i) set SP_SERVICE_successful_data_requests]]
		set SP_SERVICE_total_successful_data_request_delay [expr $SP_SERVICE_total_successful_data_request_delay + [$application_($i) set SP_SERVICE_total_successful_data_request_delay]]
		set SP_SERVICE_total_data_updates [expr $SP_SERVICE_total_data_updates + [$application_($i) set SP_SERVICE_total_data_updates]]
		set SP_SERVICE_successful_data_updates [expr $SP_SERVICE_successful_data_updates + [$application_($i) set SP_SERVICE_successful_data_updates]]
		set SP_SERVICE_total_stream_requests [expr $SP_SERVICE_total_stream_requests + [$application_($i) set SP_SERVICE_total_stream_requests]]
		set SP_SERVICE_successful_stream_requests [expr $SP_SERVICE_successful_stream_requests + [$application_($i) set SP_SERVICE_successful_stream_requests]]
		set SP_SERVICE_expected_stream_packets [expr $SP_SERVICE_expected_stream_packets + [$application_($i) set SP_SERVICE_expected_stream_packets]]
		set SP_SERVICE_received_stream_packets [expr $SP_SERVICE_received_stream_packets + [$application_($i) set SP_SERVICE_received_stream_packets]]
		set SP_SERVICE_total_replications [expr $SP_SERVICE_total_replications + [$application_($i) set SP_SERVICE_total_replications]]
		set SP_SERVICE_successful_replications [expr $SP_SERVICE_successful_replications + [$application_($i) set SP_SERVICE_successful_replications]]
		set SP_SERVICE_total_migrations [expr $SP_SERVICE_total_migrations + [$application_($i) set SP_SERVICE_total_migrations]]
		set SP_SERVICE_successful_migrations [expr $SP_SERVICE_successful_migrations + [$application_($i) set SP_SERVICE_successful_migrations]]
	}

	set avg_node_degree [expr double($SP_direct_neighbors) / double($val(nn))]
	set SP_direct_neighbors_list [lsort -integer $SP_direct_neighbors_list]
	if {[expr [llength $SP_direct_neighbors_list] % 2] == 0} then {
		set med_node_degree [expr ([lindex $SP_direct_neighbors_list [expr [llength $SP_direct_neighbors_list] / 2 - 1]] + [lindex $SP_direct_neighbors_list [expr [llength $SP_direct_neighbors_list] / 2]]) / 2.0]
	} else {
		set med_node_degree [lindex $SP_direct_neighbors_list [expr ([llength $SP_direct_neighbors_list] - 1) / 2]]
	}
	set SP_average_route [expr $SP_average_route / $val(nn)]

	if {$APP_transmittedUnicastPackets > 0} then {
		set APP_unicastPDR [expr double($APP_receivedUnicastPackets) / double($APP_transmittedUnicastPackets)]
	}
	if {$APP_transmittedBroadcastPackets > 0} then {
		set APP_broadcastPDR [expr double($APP_receivedBroadcastPackets) / double($APP_transmittedBroadcastPackets)]
	}

	if {$SP_RUDP_total_transmissions > 0} then {
		set SP_RUDP_pdr [expr double($SP_RUDP_successful_transmissions) / double($SP_RUDP_total_transmissions)]
	}

	if {$SP_SLP_total_queries > 0} then {
		set SP_SLP_recall [expr double($SP_SLP_successful_queries) / double($SP_SLP_total_queries)]
	}

	if {$SP_SERVICE_total_data_requests > 0} then {
		set SP_SERVICE_data_request_recall [expr double($SP_SERVICE_successful_data_requests) / double($SP_SERVICE_total_data_requests)]
	}
	if {$SP_SERVICE_successful_data_requests > 0} then {
		set SP_SERVICE_average_successful_data_request_delay [expr double($SP_SERVICE_total_successful_data_request_delay) / double($SP_SERVICE_successful_data_requests)]
	}
	if {$SP_SERVICE_total_data_updates > 0} then {
		set SP_SERVICE_data_update_recall [expr double($SP_SERVICE_successful_data_updates) / double($SP_SERVICE_total_data_updates)]
	}
	if {$SP_SERVICE_total_stream_requests > 0} then {
		set SP_SERVICE_stream_request_recall [expr double($SP_SERVICE_successful_stream_requests) / double($SP_SERVICE_total_stream_requests)]
	}
	if {$SP_SERVICE_expected_stream_packets > 0} then {
		set SP_SERVICE_stream_packet_recall [expr double($SP_SERVICE_received_stream_packets) / double($SP_SERVICE_expected_stream_packets)]
	}
	if {$SP_SERVICE_total_replications > 0} then {
		set SP_SERVICE_replication_success_rate [expr double($SP_SERVICE_successful_replications) / double($SP_SERVICE_total_replications)]
	}
	if {$SP_SERVICE_total_migrations > 0} then {
		set SP_SERVICE_migration_success_rate [expr double($SP_SERVICE_successful_migrations) / double($SP_SERVICE_total_migrations)]
	}



	puts [format "ERRORs:\t\t\t\t\t\t%d"					$SP_DEBUG_ERRORs]
	puts [format "WARNings:\t\t\t\t\t%d"					$SP_DEBUG_WARNs]
	puts ""
	puts [format "Nodes:\t\t\t\t\t\t%d"					$val(nn)]
	puts [format "Area:\t\t\t\t\t\t%dm x %dm"				$val(x) $val(y)]
	puts [format "Minimal node degree:\t\t\t\t%d"				$min_node_degree]
	puts [format "Average node degree:\t\t\t\t%.2f"				$avg_node_degree]
	puts [format "Median node degree:\t\t\t\t%.1f"				$med_node_degree]
	puts [format "Maximal node degree:\t\t\t\t%d"				$max_node_degree]
	puts [format "Longest route:\t\t\t\t\t%d hops"				$SP_longest_route]
	puts [format "Average route:\t\t\t\t\t%.2f hops"			$SP_average_route]
	puts ""
	puts [format "MAC transmitted packets:\t\t\t%d"				$MAC_transmittedPackets]
	puts [format "MAC received packets:\t\t\t\t%d"				$MAC_receivedPackets]
	puts [format "MAC dropped packets (capture):\t\t\t%d"			$MAC_droppedPackets_capture]
	puts [format "MAC dropped packets (collision):\t\t%d"			$MAC_droppedPackets_collision]
	puts [format "MAC dropped packets (transmit):\t\t\t%d"			$MAC_droppedPackets_transmit]
	puts [format "MAC dropped packets (bit error):\t\t%d"			$MAC_droppedPackets_bit_error]
	puts [format "MAC transmitted bytes:\t\t\t\t%d"				$MAC_transmittedBytes]
	puts [format "MAC received bytes:\t\t\t\t%d"				$MAC_receivedBytes]
	puts ""
	puts [format "APP transmitted unicast packets:\t\t%d"			$APP_transmittedUnicastPackets]
	puts [format "APP received unicast packets:\t\t\t%d"			$APP_receivedUnicastPackets]
	puts [format "APP unicast packet delivery ratio:\t\t%.2f%%"		[expr $APP_unicastPDR * 100]]
	puts [format "APP transmitted broadcast packets:\t\t%d"			$APP_transmittedBroadcastPackets]
	puts [format "APP received broadcast packets:\t\t\t%d"			$APP_receivedBroadcastPackets]
	puts [format "APP broadcast packet delivery ratio:\t\t%.2f%%"		[expr $APP_broadcastPDR * 100]]
	puts [format "APP transmitted bytes:\t\t\t\t%d"				$APP_transmittedBytes]
	puts [format "APP received bytes:\t\t\t\t%d"				$APP_receivedBytes]
	puts ""
	puts [format "SP RUDP total transmissions:\t\t\t%d"			$SP_RUDP_total_transmissions]
	puts [format "SP RUDP successful transmissions:\t\t%d"			$SP_RUDP_successful_transmissions]
	puts [format "SP RUDP packet delivery ratio:\t\t\t%.2f%%"		[expr $SP_RUDP_pdr * 100]]
	puts ""
	puts [format "SP ICMP transmitted NOPs:\t\t\t%d"			$SP_ICMP_transmitted_nops]
	puts [format "SP ICMP received NOPs:\t\t\t\t%d"				$SP_ICMP_received_nops]
	if {$SP_ICMP_transmitted_nops > 0} then {
		for {set i 0} {$i < $val(nn)} {incr i} {
			puts [format "SP ICMP NOP packet delivery ratio at node #%d (%.2f,%.2f,%.2f):\t%.2f%%" $i [$node_($i) set X_] [$node_($i) set Y_] [$node_($i) set Z_] [expr double([$application_($i) set SP_ICMP_received_nops]) / double($SP_ICMP_transmitted_nops) * 100.0]]
		}
	}
	puts ""
	puts [format "SP SLP total queries:\t\t\t\t%d"				$SP_SLP_total_queries]
	puts [format "SP SLP successful queries:\t\t\t%d"			$SP_SLP_successful_queries]
	puts [format "SP SLP recall:\t\t\t\t\t%.2f%%"				[expr $SP_SLP_recall * 100]]
	puts ""
	puts [format "SP SERVICE total data requests:\t\t\t%d"			$SP_SERVICE_total_data_requests]
	puts [format "SP SERVICE successful data requests:\t\t%d"		$SP_SERVICE_successful_data_requests]
	puts [format "SP SERVICE data request recall:\t\t\t%.2f%%"		[expr $SP_SERVICE_data_request_recall * 100]]
	puts [format "SP SERVICE total succ. data request delay:\t%.2fs"	$SP_SERVICE_total_successful_data_request_delay]
	puts [format "SP SERVICE avg. succ. data request delay:\t%.2fs"	$SP_SERVICE_average_successful_data_request_delay]
	puts [format "SP SERVICE total data updates:\t\t\t%d"			$SP_SERVICE_total_data_updates]
	puts [format "SP SERVICE successful data updates:\t\t%d"		$SP_SERVICE_successful_data_updates]
	puts [format "SP SERVICE data update recall:\t\t\t%.2f%%"		[expr $SP_SERVICE_data_update_recall * 100]]
	puts [format "SP SERVICE total stream requests:\t\t%d"			$SP_SERVICE_total_stream_requests]
	puts [format "SP SERVICE successful stream requests:\t\t%d"		$SP_SERVICE_successful_stream_requests]
	puts [format "SP SERVICE stream request recall:\t\t%.2f%%"		[expr $SP_SERVICE_stream_request_recall * 100]]
	puts [format "SP SERVICE expected stream packets:\t\t%d"		$SP_SERVICE_expected_stream_packets]
	puts [format "SP SERVICE received stream packets:\t\t%d"		$SP_SERVICE_received_stream_packets]
	puts [format "SP SERVICE stream packet recall:\t\t%.2f%%"		[expr $SP_SERVICE_stream_packet_recall * 100]]
	puts [format "SP SERVICE total replications:\t\t\t%d"			$SP_SERVICE_total_replications]
	puts [format "SP SERVICE successful replications:\t\t%d"		$SP_SERVICE_successful_replications]
	puts [format "SP SERVICE replications success rate:\t\t%.2f%%"		[expr $SP_SERVICE_replication_success_rate * 100]]
	puts [format "SP SERVICE total migrations:\t\t\t%d"			$SP_SERVICE_total_migrations]
	puts [format "SP SERVICE successful migrations:\t\t%d"			$SP_SERVICE_successful_migrations]
	puts [format "SP SERVICE migrations success rate:\t\t%.2f%%"		[expr $SP_SERVICE_migration_success_rate * 100]]

	puts ""
	puts "nodes,area_x,area_y,min_node_degree,avg_node_degree,med_node_degree,max_node_degree,SP_longest_route,SP_average_route,MAC_transmittedPackets,MAC_receivedPackets,MAC_droppedPackets_capture,MAC_droppedPackets_collision,MAC_droppedPackets_transmit,MAC_droppedPackets_bit_error,MAC_transmittedBytes,MAC_receivedBytes,APP_transmittedUnicastPackets,APP_receivedUnicastPackets,APP_unicastPDR,APP_transmittedBroadcastPackets,APP_receivedBroadcastPackets,APP_broadcastPDR,APP_transmittedBytes,APP_receivedBytes,SP_RUDP_total_transmissions,SP_RUDP_successful_transmissions,SP_RUDP_pdr,SP_ICMP_transmitted_nops,SP_ICMP_received_nops,SP_SLP_total_queries,SP_SLP_successful_queries,SP_SLP_recall,SP_SERVICE_total_data_requests,SP_SERVICE_successful_data_requests,SP_SERVICE_data_request_recall,SP_SERVICE_total_successful_data_request_delay,SP_SERVICE_average_successful_data_request_delay,SP_SERVICE_total_data_updates,SP_SERVICE_successful_data_updates,SP_SERVICE_data_update_recall,SP_SERVICE_total_stream_requests,SP_SERVICE_successful_stream_requests,SP_SERVICE_stream_request_recall,SP_SERVICE_expected_stream_packets,SP_SERVICE_received_stream_packets,SP_SERVICE_stream_packet_recall,SP_SERVICE_total_replications,SP_SERVICE_successful_replications,SP_SERVICE_replication_success_rate,SP_SERVICE_total_migrations,SP_SERVICE_successful_migrations,SP_SERVICE_migration_success_rate"
	puts "$val(nn),$val(x),$val(y),$min_node_degree,$avg_node_degree,$med_node_degree,$max_node_degree,$SP_longest_route,$SP_average_route,$MAC_transmittedPackets,$MAC_receivedPackets,$MAC_droppedPackets_capture,$MAC_droppedPackets_collision,$MAC_droppedPackets_transmit,$MAC_droppedPackets_bit_error,$MAC_transmittedBytes,$MAC_receivedBytes,$APP_transmittedUnicastPackets,$APP_receivedUnicastPackets,$APP_unicastPDR,$APP_transmittedBroadcastPackets,$APP_receivedBroadcastPackets,$APP_broadcastPDR,$APP_transmittedBytes,$APP_receivedBytes,$SP_RUDP_total_transmissions,$SP_RUDP_successful_transmissions,$SP_RUDP_pdr,$SP_ICMP_transmitted_nops,$SP_ICMP_received_nops,$SP_SLP_total_queries,$SP_SLP_successful_queries,$SP_SLP_recall,$SP_SERVICE_total_data_requests,$SP_SERVICE_successful_data_requests,$SP_SERVICE_data_request_recall,$SP_SERVICE_total_successful_data_request_delay,$SP_SERVICE_average_successful_data_request_delay,$SP_SERVICE_total_data_updates,$SP_SERVICE_successful_data_updates,$SP_SERVICE_data_update_recall,$SP_SERVICE_total_stream_requests,$SP_SERVICE_successful_stream_requests,$SP_SERVICE_stream_request_recall,$SP_SERVICE_expected_stream_packets,$SP_SERVICE_received_stream_packets,$SP_SERVICE_stream_packet_recall,$SP_SERVICE_total_replications,$SP_SERVICE_successful_replications,$SP_SERVICE_replication_success_rate,$SP_SERVICE_total_migrations,$SP_SERVICE_successful_migrations,$SP_SERVICE_migration_success_rate"
}
