/**
 * @file rudp-light_types.h
 *
 * @brief Reliable User Datagram Protocol (RUDP) - minimal implemenation (types).
 *
 * Copyright 2007-2008, Freie Universitaet Berlin. All rights reserved.
 * Please refer to the COPYING file for detailed licensing information.
 *
 * @author Georg Wittenburg
 *
 * $Id: rudp-light_types.h 2548 2008-02-13 19:20:58Z wittenbu $
 */

#ifndef RUDP_LIGHT_TYPES_H
#define RUDP_LIGHT_TYPES_H

#include "../types.h"


/*
 * RUDP transmission type.
 */

/**
 * @brief Result of a transmission.
 */
typedef struct {
	SP_errno_t			errno;		///< Error code.
	SP_callback_parameter_t		parameter;	///< Parameter for the calling function.
} SP_RUDP_transmission_result_t;

/**
 * @brief Packet transmission.
 */
typedef struct {
	SP_packet_t			packet;		///< Packet to be transmitted.
	u_int8_t			retries;	///< Number of retries left.
	bool				acked;		///< Flag whether the packet has been acknowledged.
	SP_callback_function_t		function;	///< Callback function.
	SP_RUDP_transmission_result_t*	result;		///< Pointer to result of this transmission.
} SP_RUDP_transmission_t;



/*
 * Packet types.
 */

/**
 * @brief RUDP DATA packet header.
 */
typedef struct {
	u_int8_t			orig_app;	///< Application identifier of the encapsulated packet.
	u_int8_t			orig_type;	///< Packet type of the encapsulated packet.
} SP_RUDP_DATA_packet_t;

/**
 * @brief RUDP ACK packet.
 */
typedef struct {
	SP_seqno_t			seqno;		///< Sequence number of acknowledged packet.
} SP_RUDP_ACK_packet_t;

#endif	// RUDP_LIGHT_TYPES_H
