/**
 * @file service.h
 *
 * @brief Service main file (header).
 *
 * Copyright 2007-2008, Freie Universitaet Berlin. All rights reserved.
 * Please refer to the COPYING file for detailed licensing information.
 *
 * @author Georg Wittenburg
 *
 * $Id: service.h 2548 2008-02-13 19:20:58Z wittenbu $
 */

#ifndef SERVICE_H
#define SERVICE_H

#include "../types.h"
#include "service_types.h"

#define SP_TYPE_SERVICE_CLIENT_DATA_REQUEST	0x01	///< SERVICE DATA REQUEST packet type.
#define SP_TYPE_SERVICE_CLIENT_DATA_UPDATE	0x02	///< SERVICE DATA UPDATE packet type.
#define SP_TYPE_SERVICE_CLIENT_STREAM_REQUEST	0x03	///< SERVICE STREAM REQUEST packet type.
#define SP_TYPE_SERVICE_SERVER_DATA_ITEM	0x04	///< SERVICE DATA ITEM packet type.
#define SP_TYPE_SERVICE_SERVER_STREAM_ITEM	0x05	///< SERVICE STREAM ITEM packet type.
#define SP_TYPE_SERVICE_SERVER_NOT_AVAILABLE	0x06	///< SERVICE NOT AVAILABLE packet type.
#define SP_TYPE_SERVICE_REPLICATION_REQUEST	0x07	///< SERVICE REPLICATION REQUEST packet type.
#define SP_TYPE_SERVICE_REPLICATION_DATA	0x08	///< SERVICE REPLICATION DATA packet type.
#define SP_TYPE_SERVICE_REPLICATION_ACK		0x09	///< SERVICE REPLICATION ACK packet type.
#define SP_TYPE_SERVICE_REPLICATION_NACK	0x10	///< SERVICE REPLICATION NACK packet type.

// Interface to main module.
void SERVICE_init();
void SERVICE_start();
void SERVICE_stop();
void SERVICE_finalize();
void SERVICE_recv(const SP_packet_header_t* header, const SP_packet_payload_t* payload);
bool SERVICE_command(int argc, const char*const* argv);

// Interface provided by this module (client).
void SERVICE_request_data(const char* service_id, u_int16_t offset, size_t size);
void SERVICE_update_data(const char* service_id, u_int16_t offset, size_t size, const char* data);
void SERVICE_request_stream(const char* service_id, u_int16_t offset, size_t size, SP_time_t interval, SP_time_t duration);
// Interface provided by this module (server).
void SERVICE_create_service(const char* service_id, size_t service_data_size);
void SERVICE_shutdown_service(const char* service_id);
SP_SERVICE_service_t* SERVICE_get_active_service(const char* service_id);
void SERVICE_replicate_service(const char* service_id, SP_addr_t to, SP_callback_function_t callback, SP_SERVICE_replication_result_t* result);
void SERVICE_migrate_service(const char* service_id, SP_addr_t to, SP_callback_function_t callback, SP_SERVICE_migration_result_t* result);
// Utility functions.
bool SERVICE_same_service_id(const char* service_id_1, const char* service_id_2);

#endif	// SERVICE_H
