/**
 * @file service_vars.h
 *
 * @brief Service (variables).
 *
 * Copyright 2007-2008, Freie Universitaet Berlin. All rights reserved.
 * Please refer to the COPYING file for detailed licensing information.
 *
 * @author Georg Wittenburg
 *
 * $Id: service_vars.h 2548 2008-02-13 19:20:58Z wittenbu $
 */

#ifndef SERVICE_VARS_H
#define SERVICE_VARS_H

#include "../types.h"
#include "service_types.h"
#include "../vars.h"



// Client variables.

/**
 * @brief List of cached services for which a server is known.
 */
SP_list_t* SP_SERVICE_service_cache;

/**
 * @brief Statistics counter for total number of data requests.
 */
u_int32_t SP_SERVICE_total_data_requests;

/**
 * @brief Statistics counter for number of successful data requests.
 */
u_int32_t SP_SERVICE_successful_data_requests;

/**
 * @brief Statistics counter for number of failed data requests.
 */
u_int32_t SP_SERVICE_failed_data_requests;

/**
 * @brief Statistics counter for total delay of successful data requests.
 */
double SP_SERVICE_total_successful_data_request_delay;

/**
 * @brief Statistics counter for total number of data updates.
 */
u_int32_t SP_SERVICE_total_data_updates;

/**
 * @brief Statistics counter for number of successful data updates.
 */
u_int32_t SP_SERVICE_successful_data_updates;

/**
 * @brief Statistics counter for number of failed data updates.
 */
u_int32_t SP_SERVICE_failed_data_updates;

/**
 * @brief Statistics counter for total number of stream requests.
 */
u_int32_t SP_SERVICE_total_stream_requests;

/**
 * @brief Statistics counter for number of successful stream requests.
 */
u_int32_t SP_SERVICE_successful_stream_requests;

/**
 * @brief Statistics counter for number of failed stream requests.
 */
u_int32_t SP_SERVICE_failed_stream_requests;

/**
 * @brief Statistics counter for number of expected stream packets.
 */
u_int32_t SP_SERVICE_expected_stream_packets;

/**
 * @brief Statistics counter for number of received stream packets.
 */
u_int32_t SP_SERVICE_received_stream_packets;



// Server variables.

/**
 * @brief List of currently hosted services.
 */
SP_list_t* SP_SERVICE_hosted_services;

/**
 * @brief List of currently served stream.
 */
SP_list_t* SP_SERVICE_served_streams;

/**
 * @brief List of incoming service replications.
 */
SP_list_t* SP_SERVICE_incoming_replications;

/**
 * @brief List of outgoing service replications.
 */
SP_list_t* SP_SERVICE_outgoing_replications;

/**
 * @brief Statistics counter for total number of replications.
 */
u_int32_t SP_SERVICE_total_replications;

/**
 * @brief Statistics counter for number of successful replications.
 */
u_int32_t SP_SERVICE_successful_replications;

/**
 * @brief Statistics counter for number of failed replications.
 */
u_int32_t SP_SERVICE_failed_replications;

/**
 * @brief Statistics counter for total number of migrations.
 */
u_int32_t SP_SERVICE_total_migrations;

/**
 * @brief Statistics counter for number of successful migrations.
 */
u_int32_t SP_SERVICE_successful_migrations;

/**
 * @brief Statistics counter for number of failed migrations.
 */
u_int32_t SP_SERVICE_failed_migrations;

#endif	// SERVICE_VARS_H
